@@ -1,3 +1,21 @@
+.check_data_post_boundaries <- function(data) {
+  data <- na.omit(data)
+  if (nNAlines <- length(attr(data,"na.action"))) {
+    message(paste0(nNAlines," lines of reference table still contained NA's after handling 'boundaries', and were removed. see ?handling_NAs for details."))
+  }
+  checkDegenerate <- cov(data)
+  #if ( ! anyNA(checkDegenerate)) { # cf  use="na.or.complete" => NA if no complete cases
+  eigvals <- eigen(checkDegenerate, only.values = TRUE)$values
+  if (any(abs(eigvals<1e-14))) warning(paste(
+    c("The covariance matrix of the (parameters,statistics) table seems singular,\n",
+      "implying linear relationships between the variables. Problems will likely happen.\n",
+      "Redundant variables should be eliminated."
+    )),immediate. = TRUE)
+  #}
+  data
+}
+
+
 ## fast version of dmvnorm that uses precomputed version of chol(sigma) and minimal checking
 .fast_dmvnorm <- function (x, mean, solve_t_chol_sigma, log = FALSE) {
   if (is.vector(x)) dim(x) <- c(1,length(x))
@@ -6,7 +24,7 @@
     rss <- sum(tmp^2)
   } else {
     tmp <- solve_t_chol_sigma %*% (t(x)-mean) # backsolve(chol_sigma, t(x) - mean, transpose = TRUE)
-    rss <- colSums(tmp^2)
+    rss <- .colSums(tmp^2,m=nrow(tmp), n=ncol(tmp))
   }
   logdet <- attr(solve_t_chol_sigma,"logdet")
   if (is.null(logdet)) logdet <- sum(log(diag(solve_t_chol_sigma))) # is (2017) method, precomputation is not interesting 
@@ -33,7 +51,7 @@
 }
 
 .conditional_Rmixmod <- function(jointdens, #fittedPars, 
-                                 given, expansion=Infusion.getOption("expansion")) {
+                                 given, expansion=Infusion.getOption("expansion")) {  # expansion=1 to get the conditional distribution
   nbCluster <- jointdens@nbCluster
   conddens <- jointdens
   MEAN <- conddens@parameters@mean
@@ -51,9 +69,12 @@
     conddens@parameters@mean[clu_it,] <- MEAN[clu_it,For] + sig12 %*% solve(sig22,given-mean2)
     sig11 <- COV[For,For,drop=FALSE]
     conddens@parameters@variance[[clu_it]] <- expansion* (sig11 - sig12 %*% solve(sig22,t(sig12))) 
-    condprop[clu_it] <- condprop[clu_it]*dmvnorm(t(given), # dmvnorm() tests is.vector(x) which returns FALSE if x has attributes other than names.
-                                                 mean2, sigma= sig22, log=FALSE)
+    condprop[clu_it] <- log(condprop[clu_it])+dmvnorm(t(given), # dmvnorm() tests is.vector(x) which returns FALSE if x has attributes other than names.
+                                                 mean2, sigma= sig22, log=TRUE)
   }
+  maxlog <- max(condprop)
+  condprop <- condprop-maxlog
+  condprop <- exp(condprop)
   condprop <- condprop/sum(condprop)
   conddens@parameters@proportions <- condprop
   conddens@varNames <- For
@@ -62,7 +83,8 @@
   return(conddens)
 }
 
-.conditional_mclust <- function(jointdens, fittedPars, given, expansion=Infusion.getOption("expansion")) {
+.conditional_mclust <- function(jointdens, fittedPars, given, 
+                                expansion=Infusion.getOption("expansion")) { # expansion=1 to get the conditional distribution
   nbCluster <- jointdens$G
   conddens <- jointdens
   For <- fittedPars 
@@ -84,6 +106,10 @@
     condprop[clu_it] <- condprop[clu_it]*dmvnorm(t(given), # dmvnorm() tests is.vector(x) which returns FALSE if x has attributes other than names.
                                                  mean2, sigma= sig22, log=FALSE)
   }
+  maxlog <- max(condprop)
+  condprop <- condprop-maxlog
+  condprop <- exp(condprop)
+  condprop <- condprop/sum(condprop) # normalisation (even in non-safe-log-exp version) was missing for a long time!
   conddens$parameters$pro <- condprop
   conddens$parameters$variance <- .do_call_wrap("sigma2decomp", 
                                                 list(sigma=sigma11,tol=-1), # tol to avoid a simplification in the structure of $orientation that may hit a but in mclust...
@@ -153,16 +179,31 @@
 #   return(cluObject@results[[which.min(AICs)]])
 # }
 
-.solve_t_cholfn <- function(mat) {
+.solve_t_cholfn <- function(mat, smoothing_mat=NULL) { # smoothing_mat arg added to allow optimization via iterative smmothing (which does not work)
+  if (!is.null(smoothing_mat)) mat <- mat+smoothing_mat
   resu <- solve(t(chol(mat)))
   return(structure(resu,logdet=sum(log(diag(resu)))))
 }
 
+seq_nbCluster <- function(nr) {do.call(Infusion.getOption("seq_nbCluster"), list(nr=nr))}
+
+get_nbCluster_range <- function(projdata, nr= nrow(projdata), nc=ncol(projdata), 
+                                nbCluster=seq_nbCluster(nr)) { # data must be the mixmod data
+  maxnb <- do.call(.Infusion.data$options$maxnbCluster, list(nr=nr,nc=nc))
+  if (maxnb<1L) {
+    warning("Too few simulations: Gaussian mixture modelling may fail; single cluster inferred", immediate. = TRUE)
+    return(1L)
+  } else if (any(nbCluster> maxnb)) {
+    message(paste0("Gaussian mixture modelling constrained to at most ",maxnb," clusters by the number of columns."))
+    return(nbCluster[nbCluster<=maxnb])
+  } else nbCluster 
+}
+
 infer_SLik_joint <- function(data, ## reference table ~ abc
                             stat.obs,
                             logLname=Infusion.getOption("logLname"), ## not immed useful
                             Simulate=attr(data,"Simulate"), ## may be NULL
-                            nbCluster=Infusion.getOption("nbCluster"),
+                            nbCluster=seq_nbCluster(nr=nrow(data)),
                             using=Infusion.getOption("mixturing"),
                             verbose=list(most=interactive(), ## must be explicitly set to FALSE in knitr examples
                                          pedantic=FALSE,
@@ -193,6 +234,7 @@ infer_SLik_joint <- function(data, ## reference table ~ abc
     stop("Some columns of the 'data' have identical names. Please correct.")
   }
   statNames <- names(stat.obs)
+  if (nrow(data)<20000) check_raw_stats(na.omit(data),statNames = statNames,remove = FALSE, verbose=FALSE) # silent if not problem detected, but still verbose otherwise
   for (st in statNames) {
     tabstat <- table(data[,st])
     #which_mass <- as.numeric(names(which(tabstat>1L)))
@@ -268,25 +310,24 @@ infer_SLik_joint <- function(data, ## reference table ~ abc
     if (verbose$most) cat(paste("Densities modeling:",nrow(data),"points;\n"))
     locarglist <- list(data=data[,c(fittedPars,statNames)],nbCluster=nbCluster$jointdens, 
                        seed=Infusion.getOption("mixmodSeed") , models=models)
-    checkDegenerate <- cov(locarglist$data)
-    eigvals <- eigen(checkDegenerate, only.values = TRUE)$values
-    if (any(abs(eigvals<1e-14))) warning(paste(
-      c("The covariance matrix of the (parameters,statistics) table seems singular,\n",
-        "implying linear relationships between the variables. Problems will likely happen.\n",
-        "Redundant variables should be eliminated."
-        )),immediate. = TRUE)
     if (FALSE) { ## older code; still useful for devel
       jointdens <- try(.do_call_wrap("mixmodCluster",locarglist),silent = TRUE)
       jointdens <- .get_best_mixmod_by_IC(jointdens)
     }
-    jointdens <- do.call(".densityMixmod",c(locarglist,list(stat.obs=stat.obs)))
-    if (inherits(jointdens,"try-error")) {
-      nbCluster <- eval(Infusion.getOption("nbCluster"))
-      if ( ! identical(nbCluster, locarglist$nbCluster)) {
+    jointdens <- try(do.call(".densityMixmod",c(locarglist,list(stat.obs=stat.obs))),silent=TRUE) # using seed in locarglist ie that from geoOption
+    while ((inherits(jointdens,"try-error") || anyNA(jointdens@parameters@mean)) && max(locarglist$nbCluster)>1L) {
+      # Generate better (smaller) values with two cases whether a range or a single value were tried.
+      if (length(locarglist$nbCluster)==1L) { # typical of RMSE bootstrap sample. Reduce by 1 or more
+        nbCluster <- min(locarglist$nbCluster-1L, max(get_nbCluster_range(projdata=data, nbCluster=seq(locarglist$nbCluster))))
+      } else  nbCluster <- get_nbCluster_range(projdata=data, nbCluster=seq(max(locarglist$nbCluster))) # generate a more standard range
+      if ( length(setdiff(nbCluster, locarglist$nbCluster))) { # if there are new values to try
         locarglist$nbCluster <- nbCluster
-        jointdens <- do.call(".densityMixmod",c(locarglist,list(stat.obs=stat.obs)))
-      } else stop(jointdens) # i.e stop(<error object>)
-    } ## else .densityMixmod has already run .get_best_mixmod_by_IC()  
+        jointdens <- try(do.call(".densityMixmod",c(locarglist,list(stat.obs=stat.obs))),silent=TRUE) # using seed in locarglist ie that from geoOption
+      } else break # = no more ideas to fix problem
+    } ## else .densityMixmod has already run .get_best_mixmod_by_IC()
+    if (inherits(jointdens,"try-error"))  {
+      stop(jointdens) # i.e stop(<error object>) # (But in RMSE bootstrap, new bootstrap samples may be analyzed if a few failed) 
+    } else if (anyNA(jointdens@parameters@mean)) stop("Rmixmod returned NaN's in parameter estimates.")
     # plotCluster(jointdens,data=locarglist$data,variable1="theta_p",variable2="theta") # var1: stat (prediction of projection); var2: actual param
     if (verbose$most) cat(paste(" joint density modeling:",jointdens@nbCluster,"clusters;\n"))
     if (verbose$pedantic) if (jointdens@nbCluster==max(nbCluster$jointdens)) message("Inferred # of clusters= max of allowed range.")
@@ -301,7 +342,8 @@ infer_SLik_joint <- function(data, ## reference table ~ abc
                          seed=123 , models=models)
       pardens <- try(.do_call_wrap("mixmodCluster",locarglist),silent = TRUE)
       if (inherits(pardens,"try-error")) {
-        locarglist$nbCluster <- eval(Infusion.getOption("nbCluster"))
+        # ignore any non-default infer_SLik_joint(.,nbCluster) argument
+        locarglist$nbCluster <- get_nbCluster_range(projdata=data)
         pardens <- do.call("mixmodCluster",locarglist)
       }                               
       pardens <- .get_best_mixmod_by_IC(pardens) 
@@ -344,7 +386,12 @@ infer_SLik_joint <- function(data, ## reference table ~ abc
 predict.MixmodResults <- function(object, newdata,log=TRUE, solve_t_chol_sigma_list,...) {
   Sobs_activeBoundaries <- atb <- freqs <- NULL ## FR->FR tempo
   nbCluster <- object@nbCluster
-  if (is.null(nrow(newdata)) ) newdata <- t(as.matrix(newdata)) # as.matrix keeps names
+  if (is.null(nrow(newdata)) ) {
+    newdata <- t(as.matrix(newdata)) # as.matrix keeps names
+  } else if (inherits(newdata,"data.frame")) {
+    newdata <- matrix(unlist(newdata,recursive = FALSE, use.names = FALSE), nrow=nrow(newdata),
+                      dimnames=list(NULL,colnames(newdata)) ) # newdata <- as.matrix(newdata)
+  }
   if (log) { 
     if (nbCluster>0L) {
       density <- matrix(nrow=nrow(newdata),ncol=nbCluster)
@@ -356,7 +403,7 @@ predict.MixmodResults <- function(object, newdata,log=TRUE, solve_t_chol_sigma_l
       normalizedlogs <- apply(density,2L,`-`,maxlogs) ## highest value per row is 0
       ## apply return mess again: 
       if (is.vector(normalizedlogs)) dim(normalizedlogs) <- c(1L, length(normalizedlogs))
-      mixture <- rowSums(exp(normalizedlogs)) ## exp(normalizedlogs) in (0,1); sum(exp(logLi-maxlog))= exp(-maxlog) sum(exp(logLi))= exp(-maxlog) sum(Li)
+      mixture <- .rowSums(exp(normalizedlogs),m=nrow(normalizedlogs), n=ncol(normalizedlogs)) ## exp(normalizedlogs) in (0,1); sum(exp(logLi-maxlog))= exp(-maxlog) sum(exp(logLi))= exp(-maxlog) sum(Li)
       mixture <- log(mixture) + maxlogs ## log(sum(Li))
       if ( !is.null(Sobs_activeBoundaries)) mixture <- mixture + atb*log(freqs)
     } else mixture <- atb*log(freqs)
@@ -367,37 +414,42 @@ predict.MixmodResults <- function(object, newdata,log=TRUE, solve_t_chol_sigma_l
         density[,k] <- object@parameters["proportions", k] * 
           .fast_dmvnorm(newdata, object@parameters["mean", k], solve_t_chol_sigma_list[[k]],log=log)
       }
-      mixture <- rowSums(density) ## sum(Li) 
+      mixture <- .rowSums(density,m=nrow(density), n=ncol(density)) ## sum(Li) 
       if ( !is.null(Sobs_activeBoundaries)) mixture <- mixture*freqs
     } else mixture <- freqs
   }
-  return(mixture)
+  return(mixture) # not fully protected from NaN's - for newdata *far* from the represented hypervolume.
 }
 
-.pointpredict.Rmixmod <- function(X, object, 
+.pointpredict.Rmixmod <- function(X, # parameters only 
+                                  object, 
                                   tstat.obs, # 1-row matrix as otherwise more cases should be considered for cbind'ing
-                                  log, which, ...)  {
+                                  log, 
+                                  which, # "safe" version ignores, by correcting, spuriously high logL in area of low parameter density.
+                                  solve_t_chol_sigma_lists=object$solve_t_chol_sigma_lists, # added to allow optimization via iterative smmothing (which does not work)
+                                  ...)  {
   if (is.null(dim(X))) {
-    newdata <- cbind(t(X),tstat.obs) # cbind two 1-row matrices
+    newjointX <- cbind(t(X),tstat.obs) # cbind two 1-row matrices
   } else {
     if (length(intersect(colnames(X),colnames(tstat.obs)))) stop("'X' should contain only parameters, not summary statistics")
-    newdata <- cbind(X,tstat.obs[rep(1,nrow(X)),,drop=FALSE]) # cbind two nrow(X)-row matrices
+    newjointX <- cbind(X,tstat.obs[rep(1,nrow(X)),,drop=FALSE]) # cbind two nrow(X)-row matrices
   }
   if (which!="parvaldens") {
     jointvaldens <- predict(object$jointdens,
-                            newdata=newdata,
-                            solve_t_chol_sigma_list=object$solve_t_chol_sigma_lists$jointdens,
+                            newdata=newjointX,
+                            solve_t_chol_sigma_list=solve_t_chol_sigma_lists$jointdens,
                             log=log,...)
   }
   if (which=="jointvaldens") return(jointvaldens)
-  parvaldens <- predict(object$pardens,newdata=newdata,
-                        solve_t_chol_sigma_list=object$solve_t_chol_sigma_lists$pardens,
+  parvaldens <- predict(object$pardens,
+                        newdata=newjointX, # statistics will be ignored
+                        solve_t_chol_sigma_list=solve_t_chol_sigma_lists$pardens,
                         log=log,...) 
   if (which=="parvaldens") return(parvaldens)
   # ELSE: "lik", or "safe" for safe version of "lik" using object$thr_dpar
   if (log) {
     condvaldens <- jointvaldens - parvaldens
-    if (which=="safe") condvaldens <- condvaldens + pmin(0,parvaldens-object$thr_dpar)
+    if (which=="safe") condvaldens <- condvaldens + pmin(0,parvaldens-object$thr_dpar) # decreasing returned logL when parvaldens<object$thr_dpar
   } else {
     condvaldens <- jointvaldens/parvaldens
     if (which=="safe") condvaldens <- condvaldens*pmin(1,parvaldens/object$thr_dpar)
@@ -406,10 +458,12 @@ predict.MixmodResults <- function(object, newdata,log=TRUE, solve_t_chol_sigma_l
 }
 
 
-.predict_SLik_j_mclust <- function(object, newdata, tstat.obs, log, which)  {
-  if (which!="parvaldens") jointvaldens <- predict(object$jointdens, newdata=cbind(data.frame(newdata),tstat.obs)) # ! order of elements in newdata must be that of fittedPars as in object$jointdens$data
+.predict_SLik_j_mclust <- function(object, newdata, tstat.obs, log, which, solve_t_chol_sigma_lists=object$solve_t_chol_sigma_lists)  {
+  if (which!="parvaldens") jointvaldens <- predict(object$jointdens, newdata=cbind(data.frame(newdata),tstat.obs), 
+                                                   solve_t_chol_sigma_lists=solve_t_chol_sigma_lists) # ! order of elements in newdata must be that of fittedPars as in object$jointdens$data
   if (which=="jointvaldens") return(jointvaldens)
-  parvaldens <- predict(object$pardens,newdata=newdata) 
+  parvaldens <- predict(object$pardens,newdata=newdata, 
+                        solve_t_chol_sigma_lists=solve_t_chol_sigma_lists) 
   if (which=="parvaldens") return(parvaldens)
   if (log) {
     jointvaldens <- pmax(jointvaldens, .Machine$double.xmin) # F I X M E for the value
@@ -422,64 +476,179 @@ predict.MixmodResults <- function(object, newdata,log=TRUE, solve_t_chol_sigma_l
 predict.SLik_j <- function (object, 
                             newdata, ## requests new fittedPars values! 
                             log=TRUE, 
-                            which="lik", 
+                            which="lik", # may be still preferable, to "safe", for drawing new points
                             tstat= t(attr(object$logLs,"stat.obs")), # 1-row matrix...
+                            solve_t_chol_sigma_lists=object$solve_t_chol_sigma_lists, # added to allow optimization via iterative smmothing (which does not work)
                             ...) {
   if (is.null(nrow(newdata)) ) newdata <- t(as.matrix(newdata)) # as.matrix keeps names
   if (is.null(colnames(newdata))) colnames(newdata) <- object$colTypes$fittedPars
   # not useful bc predict.MixmodResults does not handle it:
   #newdata <- cbind(newdata, t(replicate(nrow(newdata),attr(object$logLs,"stat.obs"))))
   if (inherits(object$jointdens,"densityMclust")) {
-    .predict_SLik_j_mclust(object=object, newdata=newdata, tstat.obs=tstat, log=log, which=which)
+    logl <- .predict_SLik_j_mclust(object=object, newdata=newdata, tstat.obs=tstat, log=log, which=which, solve_t_chol_sigma_lists=solve_t_chol_sigma_lists)
   } else {
-    bli <- .pointpredict.Rmixmod(X=newdata, object=object, tstat.obs=tstat, log=log, which=which)
-    names(bli) <- rownames(newdata) ## predict.glm (say) keeps names; 
+    logl <- .pointpredict.Rmixmod(X=newdata, object=object, tstat.obs=tstat, log=log, which=which, solve_t_chol_sigma_lists=solve_t_chol_sigma_lists)
+    names(logl) <- rownames(newdata) ## predict.glm (say) keeps names; 
                                     ## RMSEs computation uses names of CI bounds, to be retained here, to name its results
-    return(bli)
     # apply(newdata,1L, .pointpredict.Rmixmod, object=object, tstat.obs=tstat, log=log, which=which)
   }
+  if ( ! is.null(object$prior_logL)) logl <- logl + object$prior_logL(newdata)
+  return(logl)
 }
 
 confint.SLik_j <- function(object, parm, ## parm is the parameter which CI is sought 
-                         level=0.95, verbose=interactive(),fixed=NULL,which=c(TRUE,TRUE),...) {
+                         level=0.95, verbose=interactive(),fixed=NULL,which=c(TRUE,TRUE), 
+                         #Bartlett=0L, 
+                         ...) {
   .confintAll(object=object, parm=parm, ## parm is the parameter which CI is sought 
              givenmax = object$MSL$maxlogL,
              level= - qchisq(level,df=1)/2, ## df=1 for 1D profile; /2 to compare to logLR rather than 2 logLR  
+             #Bartlett=Bartlett,
              verbose=verbose,fixed=fixed,which=which,...)
 }
 
 refine.SLik_j <- function(object,method=NULL,...) {
   if (is.null(method)) method <- "mixmodCluster" ## no clear effect but distinct from SLik case
-  refine.default(object, surfaceData=object$logLs, method=method, ...)
+  resu <- refine.default(object, surfaceData=object$logLs, method=method, ...)
+  assign("bootreps_list", list(), envir=resu$bootLRTenv) # or reset the env by resu$bootLRTenv <- list2env(list(bootreps_list=list()))?
+  resu
 }
 
 summary.SLik_j <- summary.SLik
 
 print.SLik_j <- function(x,...) {summary.SLik_j(x,...)} 
 
+.rparam_dMixmod_around_focal <- function(object, # *dMixmod* object
+                                 focal, ## single point # recall that there exists .rparam_from_SLik_j_post(), 
+                                        # which is similar is 'object' is the posterior density, EXCEPT that it does not handle this additional constraint
+                                 solve_t_chol_sigma_list,
+                                 size) {
+  varNames <- object@varNames
+  if (is.null(dim(focal))) {
+    datanames <- names(focal) # before they are erased by the next dim() assignment (!)
+    dim(focal) <- c(1L, length(focal))
+    if (is.null(datanames)) {
+      colnames(focal) <- varNames
+    } else colnames(focal) <- datanames
+  } else if (inherits(focal,"data.frame")) {
+    focal <- matrix(unlist(focal,recursive = FALSE, use.names = FALSE), nrow=1L,
+                    dimnames=list(NULL,colnames(focal)) ) # newdata <- as.matrix(newdata)
+  }
+  if ( !is.null(Sobs_activeBoundaries <- object@Sobs_activeBoundaries)) { # if the Sobs matches some boundaries, identified in Sobs_activeBoundaries
+    # then looks whether the newdata match all of the boundaries met by Sobs
+    boundsdata <-  focal[,names(Sobs_activeBoundaries),drop=FALSE]
+    atb <- apply(boundsdata,1L,`==`,y=Sobs_activeBoundaries)
+    # If not (i.e. only partial matching) the object does not predict correctly the density => warning + heuristic patch:
+    if (!all(atb)) {warning("'predict.dMixmod' cannot compute joint out-of-boundary density from conditional at-boundary density. ")}
+    freq <- object@freq
+    freqs <- atb*freq+(1-atb)*(1-freq) ## uses 1-freq instead of the unknown density of the variable(s) in boundaries 
+    densitydata <- focal[,varNames,drop=FALSE]
+  } else if ( !is.null(simuls_activeBoundaries <- object@simuls_activeBoundaries)) {
+    ## do not reduce data in this case
+    densitydata <- focal 
+    ## only for the warning:
+    boundsdata <-  focal[,names(simuls_activeBoundaries),drop=FALSE]
+    atb <- apply(boundsdata,1L,`==`,y=simuls_activeBoundaries)
+    if (any(atb)) {
+      warning("'predict.dMixmod' cannot compute conditional at-boundary density from joint out-of-boundary density.")
+      # return value is the same as for predict(,tcstat.obs=<newdata>) 
+    }
+  } else densitydata <- focal[,varNames,drop=FALSE]
+  
+  nbCluster <- object@nbCluster
+  
+  density <- numeric(nbCluster)
+  for (k in 1:nbCluster) {
+    density[k] <- # without the 'prior' cluster proba bc aim is not to predict the cluster given these 'prior' freqs but to modify this 'prior'  
+      .fast_dmvnorm(densitydata, object@parameters["mean", k], solve_t_chol_sigma= solve_t_chol_sigma_list[[k]],log=TRUE)
+  }
+  k <- which.max(density)
+  trypoints <- rmvnorm(size, 
+                       mean=object@parameters["mean", k], 
+                       sigma= object@parameters["variance",k])
+  colnames(trypoints) <- varNames
+  trypoints
+}
+
+focal_refine <- function(object, focal, size, plotprof=TRUE, ...) {
+  fittedPars <- object$colTypes$fittedPars
+  prof <- profile(fitted=object,value=focal, return.optim=TRUE)
+  fullfocal <- c(focal,prof$par)[fittedPars]
+  posteriordens <- .conditional_Rmixmod(object$jointdens, given=attr(object$logLs,"stat.obs"), expansion=1) 
+  solve_t_chol_sigma_list <- lapply(posteriordens@parameters["variance"], .solve_t_cholfn)
+  trypoints <- .rparam_dMixmod_around_focal(posteriordens, focal = fullfocal, solve_t_chol_sigma_list=solve_t_chol_sigma_list, size=size)
+  for (vv in fittedPars) trypoints <- trypoints[trypoints[,vv]>object$LOWER[vv],,drop=FALSE]
+  for (vv in fittedPars) trypoints <- trypoints[trypoints[,vv]<object$UPPER[vv],,drop=FALSE]
+  trypoints <- data.frame(trypoints)
+  trypoints <- cbind(trypoints,object$colTypes$fixedPars) ## add fixedPars for simulation
+  trypoints <- trypoints[,object$colTypes$allPars,drop=FALSE] ## column reordering and remove polluting things
+  object <- refine(object, trypoints=trypoints, ...)
+  if (plotprof) {
+    pars <- names(focal)
+    if (length(pars)==1L) {
+      plot1Dprof(object, pars = pars, ...)
+    } else if (length(pars)==2L) {
+      plot2Dprof(object, pars = pars, ...)
+    } 
+  }
+  object
+}
+
 .rparam_from_SLik_j <- function(object, 
                                 frac, 
+                                target_size=NULL,
                                 fittedPars=NULL,level,tol=0) {
   if (is.null(fittedPars)) fittedPars <- object$colTypes$fittedPars
   CI_LRstat <- qchisq(level,df=length(fittedPars))/2 
   size_first_iter <- attr(object$logLs,"n_first_iter")
   if (is.null(size_first_iter)) size_first_iter <- length(which(object$logLs$cumul_iter==1)) ## FR->FR temporary back compat
   prev_n_iter <- max(object$logLs$cumul_iter)
-  ceil_size <- target_size <- frac*max(size_first_iter/2, size_first_iter*(prev_n_iter+1L)-nrow(object$logLs))
+  if (is.null(target_size)) target_size <- frac*max(size_first_iter/2, size_first_iter*(prev_n_iter+1L)-nrow(object$logLs))
+  #
+  ceil_size <- target_size
   if ( ! is.null(freq_good <- attr(object$logLs,"freq_good")$default)) ceil_size <- ceiling(ceil_size/freq_good)
+  #
   if (inherits(object$jointdens,"densityMclust")) {
     trypoints <- do.call("sim", c(object$jointdens[c("modelName", "parameters")],list(n=ceil_size)))
     trypoints <- trypoints[,1L+seq_len(length(fittedPars)),drop=FALSE]  
-  } else {
-    trypoints <- .simulate.MixmodResults(object$jointdens, nsim=1L, size=ceil_size, drop=TRUE) ## simulates joint
-    trypoints <- trypoints[,seq_len(length(fittedPars)),drop=FALSE] # sample in marginal distrib of params from sample in jointdens
-  } ## : samples the joint and keep only the params => distrib of trypoints=current distr of param (pardens)
+  } else if (FALSE && # inhibits block for generation of specific param points when profiling has found a new maximum (__F I X M E__)
+             (identical(object$MSL$updated_from_prof,TRUE) || # object$MSL envir recreated by user after plot1Dprof, plot2Dprof or plot() found a new max
+              ! is.null(object$MSL$init_from_prof) # only .MSL_update() run -> object$MSL$MSLE modified but envir not recreated, $init_from_prof still there.
+             )
+            ) {
+      # if we sample always in the posterior = prior*lik, we will end sampling according to the lik, so too much concentrated. 
+      size_from_pardens <- ceil_size* (1-1/object$jointdens@nbCluster)
+      trypoints <- .simulate.MixmodResults(object$jointdens, nsim=1L, size=size_from_pardens, drop=TRUE) ## simulates joint
+      trypoints <- trypoints[,seq_len(length(fittedPars)),drop=FALSE] # sample in marginal distrib of params from sample in jointdens
+      if (FALSE) {
+        postpoints <- .rparam_dMixmod_around_focal(object$jointdens, # dMixmod object
+                                                   focal=c(object$MSL$MSLE,attr(object$logLs,"stat.obs")), 
+                                                   solve_t_chol_sigma_list=object$solve_t_chol_sigma_lists$jointdens,
+                                                   size= ceil_size - nrow(trypoints))
+        # this samples the joint density around the focal point so +/- conditionally on stat.obs. 
+        # Then next line +/- samples in posterior distrib of params given stat.obs  
+        postpoints <- postpoints[,seq_len(length(fittedPars)),drop=FALSE] 
+      } else {
+        posteriordens <- .conditional_Rmixmod(object$jointdens, given=attr(object$logLs,"stat.obs"), expansion=1) 
+        solve_t_chol_sigma_list <- lapply(posteriordens@parameters["variance"], .solve_t_cholfn)
+        postpoints <- .rparam_dMixmod_around_focal(posteriordens, # dMixmod object
+                                                   focal=object$MSL$MSLE, 
+                                                   solve_t_chol_sigma_list=solve_t_chol_sigma_list,
+                                                   size= ceil_size - nrow(trypoints))
+      }
+      trypoints <- rbind(trypoints,postpoints)
+    } else {
+      trypoints <- .simulate.MixmodResults(object$jointdens, nsim=1L, size=ceil_size, drop=TRUE) ## simulates joint
+      trypoints <- trypoints[,seq_len(length(fittedPars)),drop=FALSE] # sample in marginal distrib of params from sample in jointdens
+      ## : samples the joint and keep only the params => distrib of trypoints=current distr of param (pardens)
+    }
+  
   colnames(trypoints) <- fittedPars
   for (vv in fittedPars) trypoints <- trypoints[trypoints[,vv]>object$LOWER[vv],,drop=FALSE]
   for (vv in fittedPars) trypoints <- trypoints[trypoints[,vv]<object$UPPER[vv],,drop=FALSE]
   #
-  prior <- predict(object,trypoints,which="parvaldens") ## slow (F I X M E?)
-  logLs <- predict(object,trypoints,which="lik") ## likelihood fn  slow
+  prior <- predict(object,trypoints,which="parvaldens") 
+  logLs <- predict(object,trypoints,which="lik") ## likelihood fn  
   flatnd_max <- max(logLs)-CI_LRstat ## defines an upper slice of logLs values.
   upperslice <- ( logLs> flatnd_max )
   weights <- numeric(length(logLs))
@@ -502,7 +671,7 @@ print.SLik_j <- function(x,...) {summary.SLik_j(x,...)}
   } else if (length(good)>target_size-10L) { # just missing a few points... 
     bad <- which(!wei_runif)
     order_bad <- order(weights[bad],decreasing=TRUE)
-    supplement <- bad[order_bad][seq(target_size-length(good))]
+    supplement <- bad[order_bad][seq(min(length(bad),target_size-length(good)))]
     good <- c(good,supplement)
   } else if (freq_good < 1/5) {
     bad <- which(!wei_runif)
@@ -615,13 +784,34 @@ print.SLik_j <- function(x,...) {summary.SLik_j(x,...)}
 
 profile.SLik_j <- function(fitted,...) profile.SLik(fitted=fitted,...) ## no need for two distinct methods here
 
+
+.boo_SLik_joint <- function(simul, debug_level=0L, boo, stat.obs, nbCluster_SVs) {
+  boo[] <- simul
+  if (debug_level<2L) { # return valid, NULL or try-error
+    densv <- suppressWarnings( ## suppress warnings for clustering failure
+      try(infer_SLik_joint(boo,stat.obs=stat.obs, nbCluster=nbCluster_SVs,
+                           verbose=list(most=FALSE,final=FALSE)),silent=TRUE))
+    if (inherits(densv,"try-error") && debug_level==0L) {
+      return(NULL) ## used in whichvalid <- which( ! sapply(resu,is.null)) below
+    } else return(densv)
+  } else { # return error -> useful in serial mode only
+    densv <- suppressWarnings( ## suppress warnings for clustering failure
+      infer_SLik_joint(boo,stat.obs=stat.obs, nbCluster=nbCluster_SVs,
+                       verbose=list(most=FALSE,final=FALSE)))
+    return(densv)
+  }
+}
+
+
 # Called by .RMSEwrapper.SLik_j();
-# this does not call the process-simulating function. Instead, it performs a sort of parametric bootstrap from the Gaussian mixture model.
-# Hence, it can simulate negative variance parameters and jointly simulate projected statistics from the jointdens...
+# this does not call the process-simulating function. Instead, it performs a sort of parametric bootstrap from the Gaussian mixture model,
+# i.e., it jointly simulates parameters and projected statistics from the jointdens...
+# An artefact is that it can draw meaningless values from the jointdens, e.g. negative values for variance parameters
 .boot.SLik_j <- function(object,nsim=2L, force=FALSE, verbose = TRUE, seed=NULL, 
                          parent_cores_info=NULL, # defined cluster by parent *.boot.SLik_j* calls
                          packages=attr(object$logLs,"packages"), env=attr(object$logLs,"env"), # not required if parent_cores_info is provided.
-                         cluster_args=list()
+                         cluster_args=list(),
+                         cl_seed=.update_seed(object)
                          ) {
   if (!force && nsim<2L) stop("'nsim' must be > 1")
     if (inherits(object$jointdens,"densityMclust")) {
@@ -638,29 +828,12 @@ profile.SLik_j <- function(fitted,...) profile.SLik(fitted=fitted,...) ## no nee
   
   boo <- object$logLs[,with(object$colTypes,c(fittedPars,statNames))]
   attr(boo, "allPars") <- object$colTypes$fittedPars # late 2021/01/25 correction as otherwise infer_SLik_joint() tries to use the original allPars that include fixed ones
-  it <- 0L
+  stat.obs <- attr(object$logLs,"stat.obs")
   prevmsglength <- 0L
   resu <- vector("list")
-  boo_SLik_joint <- function(simul, debug_level=0L) {
-    it <<- it+1
-    locmess <- paste("boostrap replicate",it,"of",nsim,"   ")
-    if (verbose) {prevmsglength <<- .overcat(locmess,prevmsglength = prevmsglength)}
-    boo[] <- simul
-    if (debug_level<2L) { # return valid, NULL or try-error
-      densv <- suppressWarnings( ## suppress warnings for clustering failure
-        try(infer_SLik_joint(boo,stat.obs=attr(object$logLs,"stat.obs"), nbCluster=nbCluster_SVs,
-                             verbose=list(most=FALSE,final=FALSE)),silent=TRUE))
-      if (inherits(densv,"try-error") && debug_level==0L) {
-        return(NULL) ## used in whichvalid <- which( ! sapply(resu,is.null)) below
-      } else return(densv)
-    } else { # return error -> useful in serial mode only
-      densv <- suppressWarnings( ## suppress warnings for clustering failure
-        infer_SLik_joint(boo,stat.obs=attr(object$logLs,"stat.obs"), nbCluster=nbCluster_SVs,
-                         verbose=list(most=FALSE,final=FALSE)))
-      return(densv)
-    }
-  }
+  #
   if (is.null(parent_cores_info)) {
+    cluster_args <- .set_cluster_type(cluster_args, nb_cores=NULL) # PSOCK vs FORK
     cores_info <- .init_cores(cluster_args=cluster_args)
     cl <- cores_info$cl
     if ( ! is.null(cl)) {
@@ -671,20 +844,32 @@ profile.SLik_j <- function(fitted,...) profile.SLik(fitted=fitted,...) ## no nee
       if ( ! is.null(env)) parallel::clusterExport(cl=cl, ls(env),envir=env)
     }
   } else cores_info <- parent_cores_info
-  resu <- pblapply(bootrepls,boo_SLik_joint,cl = cores_info$cl)
-  #resu <- lapply(bootrepls,boo_SLik_joint)
+  cl <- cores_info$cl
+  if ( ! is.null(cl) && ! is.null(cl_seed) ) parallel::clusterSetRNGStream(cl = cl, cl_seed)
+  resu <- pblapply(bootrepls, .boo_SLik_joint, cl = cl, boo=boo, stat.obs=stat.obs, nbCluster_SVs=nbCluster_SVs)
   whichvalid <- which( ! sapply(resu,is.null))
   resu <- resu[whichvalid]
   if (length(resu)==0L) {
     message("All bootstrap replicates failed; this suggests a problem that cannot be solved by computing more replicates.\n Trying to diagnose the problem...")
     if (cores_info$nb_cores>1L) {
       message("\nIn parallel mode, first replicate gives:")
-      resu <- pblapply(bootrepls[1],boo_SLik_joint,cl = cores_info$cl, debug_level=1)
+      resu <- pblapply(bootrepls[1],.boo_SLik_joint,cl = cores_info$cl, debug_level=1, boo=boo, stat.obs=stat.obs, nbCluster_SVs=nbCluster_SVs)
       print(resu[[1]][1])
-      message("\nTesting whether this generates an error in in serial mode:") # might not fail if pb only in parallel mode
+      if (identical(cluster_args$debug_info,TRUE)) { # ready-to-use debug code:
+        sessioninfo <- utils::sessionInfo()
+        infoptions <- Infusion.options()
+        globoptions <- options()
+        tmpname <-  .generateFileName("debug_info",ext=".rda")
+        print(paste0("Saving info in file ", tmpname,":"))
+        save(bootrepls, sessioninfo, infoptions, globoptions, object, nbCluster_SVs, verbose, prevmsglength,
+             file=tmpname)
+        # loading the .rda allows to run .boo_SLik_joint(bootrepls[[1]]) (and controlling the options, if needed)
+      } else message(paste("\nSet cluster_args$debug_info=TRUE to save some debugging info in a file\n",
+                           "and see the source of Infusion:::.boot.SLik_j() for further details about it.")) 
+      message("\nTesting whether this generates an error in serial mode:") # might not fail if pb only in parallel mode
     } else  message("\nIn serial mode, first replicate gives:") # should fail again
-    abyss <- lapply(bootrepls[1],boo_SLik_joint, debug_level=2)
-    return(NULL)
+    abyss <- lapply(bootrepls[1],.boo_SLik_joint, debug_level=2, boo=boo, stat.obs=stat.obs, nbCluster_SVs=nbCluster_SVs)
+    return(NULL) # Only if parallel failed and serial did not. 
   }
   while (length(resu)< nsim) {
     message(paste("Mixture modelling with given nbCluster failed for",nsim-length(resu),"replicate(s); drawing sample(s) again..."))
