@@ -3,16 +3,22 @@
 .Infusion.data$options <- list(## should be documented: 
                                LRthreshold= - qchisq(0.999,df=1)/2, 
                                nRealizations=1000,
-                               nbCluster= quote(seq(ceiling(nrow(data)^0.3))), # M O D I F Y doc if this is modified ! # note that functiosn such as goftest handle only this format, not a list
+                               seq_nbCluster= function(projdata, nr=nrow(projdata)) {seq(ceiling(nr^0.3))}, # M O D I F Y doc if this is modified ! # expression rather than function, so that 1:3 works.
+                               maxnbCluster= function(projdata, nr= nrow(projdata), nc=ncol(projdata)) { # 'projdata' to emphasize that these are not the user-level data (but rather, projected ones)
+                                 nr_corr <- (nr*2L) %/% 3L #4 clusters for 200 rows, 6 cols => 171 parameters sometimes fails. 
+                                 #                                      So we correct nrow adhocly so that it appear to be less than 171
+                                 floor((nr_corr+1L)/(1L+nc*(nc+1L))) # compar nrow to param of free model ((gd+1)*gd+1)*G-1 (cov mat*G + means*G + G-1 proportions)
+                               },
+                               gof_nstats_fn=function(nr,nstats) floor(nr^(1/3)),  # heuristically balancing gdim and max nbCluster
                                mixturing="Rmixmod",
-                               #using="Rmixmod", ## or ""mclust
+                               #using="Rmixmod", ## or "mclust"
                                #infer_logL_method="infer_logL_by_Rmixmod", # "infer_logL_by_mclust", ## string for clusterExport !
                                mixmodGaussianModel="Gaussian_pk_Lk_Ck", # all free vs # "Gaussian_pk_Lk_Dk_A_Dk", # shape is constant but volume and orientation are free
                                mclustModel="VVV", ## "VEV", ## equivalent to default mixmodGaussianModel
                                precision=0.1,
                                #
-                               projTrainingSize=quote(.trainsize_fn(method,stats)),
-                               knotnbr=quote(.knotnbr_fn(method,stats)),
+                               train_cP_size=quote(.train_cP_size_fn(method_string,stats)),
+                               trainingsize=quote(.trainingsize_fn(method_string,stats)),
                                projKnotNbr=1000,
                                oob=TRUE, # for .predictWrap()
                                nodesize=5L, ## that the default for regression by ranger and randomForest # F I X M E see comments where this is used
@@ -32,7 +38,6 @@
                                useEI = list(max=TRUE,profileCI=TRUE,rawCI=FALSE), ## does not seem to be used
                                CIweight=1, ## used in rparam to enhance (or not) sampling near CI bounds
                                rparamfn="rparam", ## allows choice of function
-                               fitmeCondition=quote(TRUE),
                                # Rmixmod controls
                                criterion="AIC", # criterio for selection of number of clusters (not BIC by default!)
                                #strategy=quote(.do_call_wrap("mixmodStrategy",arglist=list(),pack="Rmixmod")), ## not doc'ed...
@@ -42,7 +47,8 @@
                                # trypoints controls
                                samplingType=c(default=1,posterior=0),
                                expansion=1,
-                               maxeval=quote(10^(3+(log(length(initvec))-log(5))/log(4))) # nloptr; *modified for bobyqa (which recommends > 10 * npar^2)
+                               maxeval=quote(10^(3+(log(length(initvec))-log(5))/log(4))), # nloptr; *modified for bobyqa (which recommends > 10 * npar^2)
+                               SLRTopts=c() # c("gamma") to activate gamma correction
 )
 
 
@@ -73,7 +79,7 @@ Infusion.getOption <- function (x) {Infusion.options(x)[[1]]}
   packageStartupMessage("Infusion (version ", version, 
                         ") is loaded.", 
                         "\nType 'help(Infusion)' for a short introduction,",
-                        "\nand see https://kimura.univ-montp2.fr/~rousset/Infusion.htm",
+                        "\nand see https://gitlab.mbb.univ-montp2.fr/francois/Infusion",
                         "\nfor more documentation.")
 }
 
