@@ -74,8 +74,8 @@ if (FALSE) { # example of distinct trainsample
 
 
 if (FALSE) { # example of reprojecting accumulated simulations
-  remufit <- project("mu",stats=c("mean","var"),data=slik_j$raw_data, method="REML", trainingsize=400, knotnbr=1000)
-  res2fit <- project("s2",stats=c("mean","var"),data=slik_j$raw_data, method="REML", trainingsize=400, knotnbr=1000)
+  remufit <- project("mu",stats=c("mean","var"),data=slik_j$raw_data, method="REML", train_cP_size=400, trainingsize=1000)
+  res2fit <- project("s2",stats=c("mean","var"),data=slik_j$raw_data, method="REML", train_cP_size=400, trainingsize=1000)
   reprojectors <- list("MEAN"=remufit,"VAR"=res2fit)
   reprojectors <- list2env(reprojectors)
   recorrSobs <- project(Sobs,projectors=reprojectors)
